@@ -20,7 +20,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umfuse.c,v 1.41 2007/01/18 12:33:03 garden Exp $
+ *   $Id: umfuse.c 465 2008-04-19 06:21:14Z rd235 $
  *
  */   
 #include <unistd.h>
@@ -57,6 +57,7 @@
 #include <sys/statvfs.h>
 #include <config.h>
 
+#define UMFUSE_SERVICE_CODE 0x01
 #define FUSE_SUPER_MAGIC 0x65735546
 
 /* Enable Experimental code */
@@ -119,7 +120,7 @@ struct umfuse_dirent64 {
 
 struct umdirent {
 	struct umfuse_dirent64 de;
-	unsigned short d_reclen32;
+	//unsigned short d_reclen32;
 	struct umdirent *next;
 };
 
@@ -540,6 +541,7 @@ static void *startmain(void *vsmo)
 	char *opts;
 	int newargc;
 	char **newargv;
+	char **newnewargv;
 	int i;
 	if (pmain == NULL) {
 		GMESSAGE("%s", dlerror());
@@ -560,11 +562,15 @@ static void *startmain(void *vsmo)
 		fflush(stderr);		
 	}
 
-	if (pmain(newargc,newargv) != 0)
-		umfuse_abort(psmo->new->fuse);
-	for (i=0;i<newargc;i++)
-		free(newargv[i]);
-	free(newargv);
+	/* some modules could change argv! */
+	if ((newnewargv=malloc(newargc * sizeof (char *))) != NULL) {
+		for (i=0;i<newargc;i++) 
+			newnewargv[i]=newargv[i];
+		if (pmain(newargc,newnewargv) != 0)
+			umfuse_abort(psmo->new->fuse);
+		free(newnewargv);
+	}
+	fusefreearg(newargc,newargv);
 	pthread_exit(NULL);
 	return NULL;
 }
@@ -924,7 +930,7 @@ static int umfusefilldir(fuse_dirh_t h, const char *name, int type, ino_t ino)
 		new->de.d_type=type;
 		new->de.d_name=strdup(name);
 		new->de.d_reclen=WORDALIGN(SIZEDIRENT64NONAME+strlen(name)+1);
-		new->d_reclen32=WORDALIGN(SIZEDIRENT32NONAME+strlen(name)+1);
+		//new->d_reclen32=WORDALIGN(SIZEDIRENT32NONAME+strlen(name)+1);
 
 		/* virtualize the offset on a real file, 64bit ino+16len+8namlen+8type */
 		new->de.d_off=h->offset=h->offset+WORDALIGN(12+strlen(name));
@@ -953,7 +959,7 @@ static int umfusefillreaddir(void *buf, const char *name, const struct stat *stb
 		}
 		new->de.d_name=strdup(name);
 		new->de.d_reclen=WORDALIGN(SIZEDIRENT64NONAME+strlen(name)+1);
-		new->d_reclen32=WORDALIGN(SIZEDIRENT32NONAME+strlen(name)+1);
+		//new->d_reclen32=WORDALIGN(SIZEDIRENT32NONAME+strlen(name)+1);
 		/* virtualize the offset on a real file, 64bit ino+16len+8namlen+8type */
 		new->de.d_off=h->offset=h->offset+WORDALIGN(12+strlen(name));
 		if (h->tail==NULL) {
@@ -999,6 +1005,7 @@ static void umcleandirinfo(struct umdirent *tail)
 	}
 }
 
+#if 0
 static long umfuse_getdents(unsigned int fd, struct dirent *dirp, unsigned int count)
 {
 	if (filetab[fd]==NULL) {
@@ -1038,6 +1045,7 @@ static long umfuse_getdents(unsigned int fd, struct dirent *dirp, unsigned int c
 		return curoffs;
 	}
 }
+#endif
 
 static long umfuse_getdents64(unsigned int fd, struct dirent64 *dirp, unsigned int count)
 {
@@ -1110,7 +1118,7 @@ static epoch_t fuse_path(int type, void *arg)
 			return FALSE;
 	} else if (type == CHECKFSTYPE) {
 		char *path=arg;
-		return (strncmp(path,"umfuse",6) == 0);/* a path with no leading / is a filesystemtype */
+		return (strncmp(path,"umfuse",6) == 0);
 	} else {
 		return FALSE;
 	}
@@ -1502,6 +1510,7 @@ static long umfuse_fstat64(int fd, struct stat64 *buf64)
 	return rv;
 }
 
+#if 0
 static long umfuse_stat(char *path, struct stat *buf)
 {
 	return common_stat(searchcontext(path,SUBSTR),path,buf,1);
@@ -1511,6 +1520,7 @@ static long umfuse_lstat(char *path, struct stat *buf)
 {
 	return common_stat(searchcontext(path,SUBSTR),path,buf,1);
 }
+#endif
 
 static long umfuse_stat64(char *path, struct stat64 *buf64)
 {
@@ -2287,7 +2297,7 @@ init (void)
 {
 	GMESSAGE("umfuse init");
 	s.name="umfuse fuse ";
-	s.code=0x01;
+	s.code=UMFUSE_SERVICE_CODE;
 	s.checkfun=fuse_path;
 	pthread_key_create(&context_key,NULL);
 	s.syscall=(sysfun *)calloc(scmap_scmapsize,sizeof(sysfun));
@@ -2360,6 +2370,7 @@ fini (void)
 	free(s.syscall);
 	free(s.socket);
 	forallfusetabdo(contextclose);
+	pthread_key_delete(context_key);
 	GMESSAGE("umfuse fini");
 }
 
